Package: princurve
Version: 2.1.6
Title: Fit a Principal Curve in Arbitrary Dimension
Authors@R: c(
    person("Trevor", "Hastie", email = "trevor@research.att.com", role = "aut"),
    person("Andreas", "Weingessel", email = "Andreas.Weingessel@ci.tuwien.ac.at", role = "aut"),
    person(
      "Kurt", "Hornik", 
      email = "kurt.hornik@wu.ac.at", 
      role = "aut",
      comment = c(ORCID = "0000-0003-4198-9911")
    ),
    person(
      "Henrik", "Bengtsson", 
      email = "henrikb@braju.com", 
      role = "ctb",
      comment = c(github = "HenrikBengtsson")
    ),
    person(
      "Robrecht", "Cannoodt", 
      email = "rcannood@gmail.com", 
      role = c("aut", "cre"),
      comment = c(ORCID = "0000-0003-3641-729X", github = "rcannood")
    ))
Description: Fitting a principal curve to a data matrix in arbitrary dimensions. 
  Hastie and Stuetzle (1989) <doi:10.2307/2289936>.
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.0)
Imports: stats, graphics, grDevices, Rcpp
Suggests: devtools, testthat
LinkingTo: Rcpp
NeedsCompilation: yes
RoxygenNote: 7.1.1
URL: https://github.com/rcannood/princurve
BugReports: https://github.com/rcannood/princurve/issues
Collate: 'RcppExports.R' 'bias_correct_curve.R' 'deprecated.R'
        'package.R' 'periodic_lowess.R' 'smoother_functions.R'
        'principal_curve.R' 'start_circle.R'
Packaged: 2021-01-17 18:16:00 UTC; rcannood
Author: Trevor Hastie [aut],
  Andreas Weingessel [aut],
  Kurt Hornik [aut] (<https://orcid.org/0000-0003-4198-9911>),
  Henrik Bengtsson [ctb] (HenrikBengtsson),
  Robrecht Cannoodt [aut, cre] (<https://orcid.org/0000-0003-3641-729X>,
    rcannood)
Maintainer: Robrecht Cannoodt <rcannood@gmail.com>
Repository: CRAN
Date/Publication: 2021-01-18 16:40:06 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 10:43:02 UTC; unix
Archs: princurve.so.dSYM

Package: princurve
Version: 2.1.6
Title: Fit a Principal Curve in Arbitrary Dimension
Authors@R: c(
    person("Trevor", "Hastie", email = "trevor@research.att.com", role = "aut"),
    person("Andreas", "Weingessel", email = "Andreas.Weingessel@ci.tuwien.ac.at", role = "aut"),
    person(
      "Kurt", "Hornik", 
      email = "kurt.hornik@wu.ac.at", 
      role = "aut",
      comment = c(ORCID = "0000-0003-4198-9911")
    ),
    person(
      "Henrik", "Bengtsson", 
      email = "henrikb@braju.com", 
      role = "ctb",
      comment = c(github = "HenrikBengtsson")
    ),
    person(
      "Robrecht", "Cannoodt", 
      email = "rcannood@gmail.com", 
      role = c("aut", "cre"),
      comment = c(ORCID = "0000-0003-3641-729X", github = "rcannood")
    ))
Description: Fitting a principal curve to a data matrix in arbitrary dimensions. 
  Hastie and Stuetzle (1989) <doi:10.2307/2289936>.
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.0)
Imports: stats, graphics, grDevices, Rcpp
Suggests: devtools, testthat
LinkingTo: Rcpp
NeedsCompilation: yes
RoxygenNote: 7.1.1
URL: https://github.com/rcannood/princurve
BugReports: https://github.com/rcannood/princurve/issues
Collate: 'RcppExports.R' 'bias_correct_curve.R' 'deprecated.R'
        'package.R' 'periodic_lowess.R' 'smoother_functions.R'
        'principal_curve.R' 'start_circle.R'
Packaged: 2021-01-17 18:16:00 UTC; rcannood
Author: Trevor Hastie [aut],
  Andreas Weingessel [aut],
  Kurt Hornik [aut] (<https://orcid.org/0000-0003-4198-9911>),
  Henrik Bengtsson [ctb] (HenrikBengtsson),
  Robrecht Cannoodt [aut, cre] (<https://orcid.org/0000-0003-3641-729X>,
    rcannood)
Maintainer: Robrecht Cannoodt <rcannood@gmail.com>
Repository: CRAN
Date/Publication: 2021-01-18 16:40:06 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 16:22:33 UTC; unix
Archs: princurve.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(lines,principal.curve)
S3method(lines,principal_curve)
S3method(plot,principal.curve)
S3method(plot,principal_curve)
S3method(points,principal.curve)
S3method(points,principal_curve)
export(get.lam)
export(principal.curve)
export(principal_curve)
export(project_to_curve)
export(whiskers)
import(Rcpp)
importFrom(grDevices,extendrange)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(stats,approx)
importFrom(stats,lowess)
importFrom(stats,predict)
importFrom(stats,smooth.spline)
importFrom(stats,var)
useDynLib(princurve)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# princurve

[![CRAN
Status](https://www.r-pkg.org/badges/version/princurve)](https://cran.r-project.org/package=princurve)
[![CRAN
Downloads](https://cranlogs.r-pkg.org/badges/princurve)](https://cran.r-project.org/package=princurve)
[![DOI](https://zenodo.org/badge/125849601.svg)](https://zenodo.org/badge/latestdoi/125849601)  
[![R-CMD-check](https://github.com/rcannood/princurve/workflows/R-CMD-check/badge.svg)](https://github.com/rcannood/princurve/actions?query=workflow%3AR-CMD-check)
[![Coverage
Status](https://codecov.io/gh/rcannood/princurve/branch/master/graph/badge.svg)](https://codecov.io/gh/rcannood/princurve?branch=master)

Fitting a principal curve to a data matrix in arbitrary dimensions. A
principal curve is a smooth curve passing through the middle of a
multidimensional dataset. This package is an R/C++ reimplementation of
the S/Fortran code provided by Trevor Hastie, with multiple performance
tweaks.

## Example

Usage of princurve is demonstrated with a toy dataset.

``` r
t <- runif(100, -1, 1)
x <- cbind(t, t ^ 2) + rnorm(200, sd = 0.05)
colnames(x) <- c("dim1", "dim2")

plot(x)
```

![](man/figures/README_example-1.png)<!-- -->

A principal curve can be fit to the data as follows:

``` r
library(princurve)
fit <- principal_curve(x)
plot(fit); whiskers(x, fit$s, col = "gray")
```

![](man/figures/README_princurve-1.png)<!-- -->

See `?principal_curve` for more information on how to use the
`princurve` package.

## Latest changes

Check out `news(package = "princurve")` or [NEWS.md](NEWS.md) for a full
list of changes.

<!-- This section gets automatically generated from NEWS.md -->

### Recent changes in princurve 2.1.6 (2021-01-17)

-   BUG FIX `project_to_curve()`: Return error message when `x` or `s`
    contain insufficient rows.

-   BUG FIX unit tests: Switch from `svg()` to `pdf()` as support for
    `svg()` might be optional.

### Recent changes in princurve 2.1.5 (2020-08-13)

-   BUG FIX `project_to_curve()`: Fix pass-by-reference bug, issue \#33.
    Thanks to @szcf-weiya for detecting and fixing this bug!

## References

Hastie, T. and Stuetzle, W., [Principal
Curves](https://www.jstor.org/stable/2289936), JASA, Vol. 84, No. 406
(Jun., 1989), pp. 502-516, DOI:
[10.2307/2289936](https://doi.org/10.2307/2289936)
([PDF](https://web.stanford.edu/~hastie/Papers/principalcurves.pdf))

#' Generate circle as initial curve
#'
#' The starting circle is defined in the first two dimensions,
#' and has zero values in all other dimensions.
#'
#' @param x The data for which to generate the initial circle
#'
#' @examples
#' \dontrun{
#' x <- cbind(
#'   rnorm(100, 1, .2),
#'   rnorm(100, -5, .2),
#'   runif(100, 1.9, 2.1),
#'   runif(100, 2.9, 3.1)
#' )
#' circ <- start_circle(x)
#' plot(x)
#' lines(circ)
#' }
start_circle <- function(x) {
  n_points <- 360 # number of points to generate for the initial_s

  xbar <- colMeans(x)
  radius <- colMeans(abs(sweep(x, 2, xbar)))
  theta <- pi * seq_len(n_points) * 2 / n_points

  initial_s <- cbind(radius[[1]] * sin(theta) + xbar[[1]], radius[[2]] * cos(theta) + xbar[[2]])

  if (ncol(x) > 2) {
    initial_s <- cbind(initial_s, matrix(rep(xbar[-1:-2], n_points), nrow = n_points, byrow = TRUE))
  }

  project_to_curve(x = x, s = initial_s, stretch = 0)
}

context("Testing start_circle")

test_start_circle_output <- function(x_orig) {
  xbar_orig <- apply(x_orig, 2, mean)

  fit <- start_circle(x_orig)
  x_new <- fit$s
  xbar_new <- apply(x_new, 2, mean)
  radius_new <- colMeans(abs(sweep(x_new, 2, xbar_new)))[1:2]

  expect_equal(dim(x_new), dim(x_orig))
  expect_equal(rownames(x_new), rownames(x_orig))
  expect_equal(colnames(x_new), colnames(x_orig))
  expect_lte(sum((xbar_orig - xbar_new)^2), 1e-2)

  # check whether bar and radius update correctly after translation
  fit2 <- start_circle(x_orig + 10)
  x_new2 <- fit2$s
  xbar_new2 <- apply(x_new2, 2, mean)
  radius_new2 <- colMeans(abs(sweep(x_new2, 2, xbar_new2)))[1:2]

  expect_lte(sum((abs(xbar_new2 - xbar_new) - 10)^2), 1e-10)
  expect_lte(sum((radius_new2 - radius_new)^2), 1e-10)

  # check whether bar and radius update correctly after scaling
  fit3 <- start_circle(x_orig * 10)
  x_new3 <- fit3$s
  xbar_new3 <- apply(x_new3, 2, mean)
  radius_new3 <- colMeans(abs(sweep(x_new3, 2, xbar_new3)))[1:2]

  expect_lte(sum((abs(xbar_new3 / xbar_new) - 10)^2), 1e-10)
  expect_lte(sum((abs(radius_new3 / radius_new) - 10)^2), 1e-10)
}

test_that("Testing start_circle", {
  x_orig <- cbind(
    rnorm(100, 0, 1),
    rnorm(100, 0, 1)
  )

  test_start_circle_output(x_orig)

  colnames(x_orig) <- paste0("Dim", seq_len(ncol(x_orig)))
  test_start_circle_output(x_orig)

  rownames(x_orig) <- paste0("Sample", seq_len(nrow(x_orig)))
  test_start_circle_output(x_orig)

  colnames(x_orig) <- NULL
  test_start_circle_output(x_orig)
})


test_that("Testing start_circle", {
  x_orig <- cbind(
    rnorm(404, 1, .2),
    rnorm(404, -5, .2),
    runif(404, 1.9, 2.1),
    runif(404, 2.9, 3.1)
  )

  test_start_circle_output(x_orig)

  colnames(x_orig) <- paste0("Dim", seq_len(ncol(x_orig)))
  test_start_circle_output(x_orig)

  rownames(x_orig) <- paste0("Sample", seq_len(nrow(x_orig)))
  test_start_circle_output(x_orig)

  colnames(x_orig) <- NULL
  test_start_circle_output(x_orig)
})

#' @importFrom stats approx
bias_correct_curve <- function(x, pcurve, ...) {
  # bias correction, as suggested by Jeff Banfield
  ones <- rep(1, ncol(x))
  sbar <- colMeans(pcurve$s)
  ray <- drop(sqrt(((x - pcurve$s)^2) %*% ones))
  dist1 <- (scale(x, sbar, FALSE)^2) %*% ones
  dist2 <- (scale(pcurve$s, sbar, FALSE)^2) %*% ones
  sign <- 2 * as.double(dist1 > dist2) - 1
  ray <- sign * ray

  ploess <- periodic_lowess(pcurve$lambda, ray, ...)
  sray <- stats::approx(
    ploess$x,
    ploess$y,
    pcurve$lambda,
    ties = "ordered"
  )$y

  pcurve$s <- pcurve$s + (abs(sray) / ray) * (x - pcurve$s)
  project_to_curve(x, pcurve$s[pcurve$ord, , drop = FALSE], stretch = 0)
}

#' @importFrom stats lowess approx
periodic_lowess <- function(x, y, f = 0.59999999999999998, ...) {
  n <- length(x)
  o <- order(x)
  r <- range(x)

  d <- diff(r) / (length(unique(x)) - 1)
  xr <- x[o[1 : (n/2)]] - r[1] + d + r[2]
  xl <- x[o[(n/2) : n]] - r[2] - d + r[1]
  yr <- y[o[1 : (n/2)]]
  yl <- y[o[(n/2) : n]]

  xnew <- c(xl, x, xr)
  ynew <- c(yl, y, yr)

  f <- f / 2
  fit <- stats::lowess(xnew, ynew, f = f, ...)
  stats::approx(fit$x, fit$y, x[o], rule = 2, ties = "ordered")
}

#' Deprecated functions
#'
#' This function is deprecated, please use
#' \code{\link{principal_curve}} and \code{\link{project_to_curve}} instead.
#'
#' @param ... Catch-all for old parameters.
#'
#' @rdname deprecated
#'
#' @export
principal.curve <- function(...) {
  .Defunct("principal_curve", package = "princurve")
}


#' @rdname deprecated
#' @export
lines.principal.curve <- function(...) {
  .Defunct("lines.principal_curve", package = "princurve")
}

#' @rdname deprecated
#' @export
plot.principal.curve <- function(...) {
  .Defunct("plot.principal_curve", package = "princurve")
}

#' @rdname deprecated
#' @export
points.principal.curve <- function(...) {
  .Defunct("points.principal_curve", package = "princurve")
}

#' @rdname deprecated
#' @export
get.lam <- function(...) {
  .Defunct("project_to_curve", package = "princurve")
}

context("Comparing principal_curve and project_to_curve to legacy package")

skip_on_cran()

if (!requireNamespace("princurvelegacy", quietly = TRUE)) {
  devtools::install_github("rcannood/princurve@legacy")
}

for (i in seq_len(10)) {
  test_that(paste0("Directly compare principal_curve against legagy, run ", i), {
    x <- matrix(runif(1000), ncol = 10)

    fit1 <- principal_curve(x)
    fit2 <- princurvelegacy::principal.curve(x)

    expect_equivalent(fit1$s, fit2$s, tolerance = .001)
    expect_gte(cor(order(fit1$ord), order(fit2$tag)), .99)
    expect_equivalent(fit1$lambda, fit2$lambda, tolerance = .001)
    expect_equivalent(fit1$dist, fit2$dist, tolerance = .001)
    expect_equal(fit1$converged, fit2$converged)
    expect_equal(fit1$num_iterations, fit2$nbrOfIterations)
  })
}

for (i in seq_len(10)) {
  test_that(paste0("Directly compare project_to_curve against legagy, run ", i), {
    x <- matrix(runif(1000), ncol = 10)
    s <- matrix(runif(100), ncol = 10)

    fit1 <- project_to_curve(x, s)
    fit2 <- princurvelegacy::get.lam(x, s)

    expect_equivalent(fit1$s, fit2$s, tolerance = .001)
    expect_gte(cor(order(fit1$ord), order(fit2$tag)), .99)
    expect_equivalent(fit1$lambda, fit2$lambda, tolerance = .001)
    expect_equivalent(fit1$dist, fit2$dist, tolerance = .001)
  })
}

context("Legacy functions have been properly deprecated")

test_that("Test whether legacy functions have been deprecated", {
  expected_class <- ifelse(getRversion() >= "3.6", "defunctError", "simpleError")
  expect_condition(principal.curve(), "is defunct", class = expected_class)
  expect_condition(lines.principal.curve(), "is defunct", class = expected_class)
  expect_condition(plot.principal.curve(), "is defunct", class = expected_class)
  expect_condition(points.principal.curve(), "is defunct", class = expected_class)
  expect_condition(get.lam(), "is defunct", class = expected_class)
})

#' Smoother functions
#'
#' Each of these functions have an interface \code{function(lambda, xj, ...)}, and
#' return smoothed values for xj. The output is expected to be ordered along an ordered lambda.
#' This means that the following is true:
#' \preformatted{
#' x <- runif(100)
#' y <- runif(100)
#' ord <- sample.int(100)
#' sfun <- smoother_functions[[1]]
#' all(sfun(x, y) == sfun(x[ord], y[ord]))
#' }
smoother_functions <- list(
  smooth_spline = function(lambda, xj, ..., df = 5) {
    ord <- order(lambda)
    lambda <- lambda[ord]
    xj <- xj[ord]
    fit <- stats::smooth.spline(lambda, xj, ..., df = df, keep.data = FALSE)
    stats::predict(fit, x = lambda)$y
  },

  lowess = function(lambda, xj, ...) {
    stats::lowess(lambda, xj, ...)$y
  },

  periodic_lowess = function(lambda, xj, ...) {
    periodic_lowess(lambda, xj, ...)$y
  }
)

#' Fits a Principal Curve in Arbitrary Dimension
#'
#' @name princurve-package
#' @aliases princurve-package princurve
#' @docType package
#'
#' @import Rcpp
#'
#' @seealso \code{\link{principal_curve}}, \code{\link{project_to_curve}}
#'
#' @references
#'  Hastie, T. and Stuetzle, W.,
#'  \href{https://www.jstor.org/stable/2289936}{Principal Curves},
#'  JASA, Vol. 84, No. 406 (Jun., 1989), pp. 502-516,
#'  DOI: \href{https://doi.org/10.2307/2289936}{10.2307/2289936}
#'  (\href{https://web.stanford.edu/~hastie/Papers/principalcurves.pdf}{PDF}).
#'
#'  See also Banfield and Raftery (JASA, 1992).
#'
#' @keywords regression smooth nonparametric
#'
#' @useDynLib princurve
NULL

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Project a set of points to the closest point on a curve
#'
#' Finds the projection index for a matrix of points \code{x}, when
#' projected onto a curve \code{s}. The curve need not be of the same
#' length as the number of points.
#'
#' @param x a matrix of data points.
#' @param s a parametrized curve, represented by a polygon.
#' @param stretch A stretch factor for the endpoints of the curve,
#'   allowing the curve to grow to avoid bunching at the end.
#'   Must be a numeric value between 0 and 2.
#'
#' @return A structure is returned which represents a fitted curve.  It has components
#'   \item{s}{The fitted points on the curve corresponding to each point \code{x}}
#'   \item{ord}{the order of the fitted points}
#'   \item{lambda}{The projection index for each point}
#'   \item{dist}{The total squared distance from the curve}
#'   \item{dist_ind}{The squared distances from the curve to each of the respective points}
#'
#' @seealso \code{\link{principal_curve}}
#'
#' @keywords regression smooth nonparametric
#'
#' @export
project_to_curve <- function(x, s, stretch = 2) {
    .Call('_princurve_project_to_curve', PACKAGE = 'princurve', x, s, stretch)
}

context("Testing project_to_curve")

# helper function for validating project_to_curve output
test_projection <- function(x, s, stretch, fit) {
  # check names
  expect_equal(names(fit), c("s", "ord", "lambda", "dist_ind", "dist"))
  expect_equal(rownames(fit$s), rownames(x))
  expect_equal(colnames(fit$s), colnames(x))
  expect_equal(names(fit$lambda), rownames(x))
  expect_equal(names(fit$dist_ind), rownames(x))
  expect_equal(names(fit$ord), NULL)
  expect_equal(names(fit$dist), NULL)

  # check lambda
  sord <- fit$s[fit$ord,]
  slam <- cumsum(c(0, sqrt(rowSums((sord[-nrow(sord),] - sord[-1,])^2))))
  expect_lte(sum((slam - fit$lambda[fit$ord])^2), 1e-10)

  # check dist_ind
  dist_ind <- rowSums((fit$s - x)^2)
  expect_lte(sum((dist_ind - fit$dist_ind)^2), 1e-10)

  # check dist
  dist <- sum(dist_ind)
  expect_lte(sum((dist - fit$dist)^2), 1e-10)

  # reproject
  fit2 <- project_to_curve(fit$s, fit$s, stretch = 0)
  expect_lte(sum((fit2$s - fit$s)^2), 1e-10)
}

# test data
z <- seq(-1, 1, length.out = 100)
s <- cbind(z, z^2, z^3, z^4)
x <- s + rnorm(length(s), mean = 0, sd = .005)
ord <- sample.int(nrow(x))


test_that("Testing project_to_curve", {
  fit <- project_to_curve(
    x = x,
    s = s,
    stretch = 0
  )

  test_projection(x, s, stretch = 0, fit)

  expect_gte(cor(as.vector(fit$s), as.vector(s)), .99)
  expect_gte(cor(fit$ord, seq_len(100)), .99)
})


test_that("Checking whether project_to_curve retains dimnames", {
  colnames(x) <- paste0("Comp", seq_len(ncol(x)))
  fit <- project_to_curve(x = x, s = s, stretch = 0)
  test_projection(x, s, stretch = 0, fit)

  rownames(x) <- paste0("Sample", seq_len(nrow(x)))
  fit <- project_to_curve(x = x, s = s, stretch = 0)
  test_projection(x, s, stretch = 0, fit)

  colnames(x) <- NULL
  fit <- project_to_curve(x = x, s = s, stretch = 0)
  test_projection(x, s, stretch = 0, fit)
})


test_that("Testing project_to_curve with shuffled order", {
  fit <- project_to_curve(
    x = x[ord,],
    s = s,
    stretch = 0
  )

  test_projection(x[ord,], s, stretch = 0, fit)

  expect_gte(cor(as.vector(fit$s[fit$ord,]), as.vector(s)), .99)
  expect_gte(cor(order(fit$ord), ord), .99)
})

test_that("Values are more or less correct", {
  constant_s <- matrix(c(-1, 1, .1, .1, .2, .2, .3, .3), nrow = 2, byrow = FALSE)
  x[,1] <- z

  fit <- project_to_curve(
    x = x,
    s = constant_s,
    stretch = 0
  )

  test_projection(x, constant_s, stretch = 0, fit)

  expect_true(all(abs(fit$s[,1] - x[,1]) < 1e-6))
  expect_true(all(abs(fit$s[,2] - .1) < 1e-6))
  expect_true(all(abs(fit$s[,3] - .2) < 1e-6))
  expect_true(all(abs(fit$s[,4] - .3) < 1e-6))
  expect_equal(fit$ord, seq_along(z))
  expect_true(all(abs(fit$lambda - seq(0, 2, length.out = 100)) < 1e-6))

  dist_ind <-
    (x[,2] - .1)^2 +
    (x[,3] - .2)^2 +
    (x[,4] - .3)^2

  expect_true(all(abs(fit$dist_ind - dist_ind) < 1e-10))

  expect_true(abs(sum(dist_ind) - fit$dist) < 1e-10)
})



test_that("Values are more or less correct, with stretch = 2 and a given ord", {
  constant_s <- matrix(c(-.9, .9, .1, .1, .2, .2, .3, .3), nrow = 2, byrow = FALSE)
  x[,1] <- z

  fit <- project_to_curve(
    x = x,
    s = constant_s,
    stretch = 2
  )

  test_projection(x, constant_s, stretch = 0, fit)

  expect_true(all(abs(fit$s[,1] - x[,1]) < 1e-6))
  expect_true(all(abs(fit$s[,2] - .1) < 1e-6))
  expect_true(all(abs(fit$s[,3] - .2) < 1e-6))
  expect_true(all(abs(fit$s[,4] - .3) < 1e-6))
  expect_equal(fit$ord, seq_along(z))
  expect_true(all(abs(fit$lambda - seq(0, 2, length.out = 100)) < 1e-3))

  dist_ind <-
    (x[,2] - .1)^2 +
    (x[,3] - .2)^2 +
    (x[,4] - .3)^2

  expect_true(all(abs(fit$dist_ind - dist_ind) < 1e-10))

  expect_true(abs(sum(dist_ind) - fit$dist) < 1e-10)
})



test_that("Values are more or less correct, without stretch", {
  cut <- 0.89898990
  constant_s <- matrix(c(-cut, cut, .1, .1, .2, .2, .3, .3), nrow = 2, byrow = FALSE)
  x[,1] <- z

  fit <- project_to_curve(
    x = x,
    s = constant_s,
    stretch = 0
  )

  test_projection(x, constant_s, stretch = 2, fit)

  f <- z < -cut | z > cut

  expect_true(all(abs(fit$s[!f,1] - x[!f,1]) < 1e-6))
  expect_false(any(abs(fit$s[f,1] - x[f,1]) < 1e-6))

  expect_true(all(abs(fit$s[,2] - .1) < 1e-6))
  expect_true(all(abs(fit$s[,3] - .2) < 1e-6))
  expect_true(all(abs(fit$s[,4] - .3) < 1e-6))
  expect_true(cor(fit$ord, seq_along(z)) > cut)

  lambda <- apply(fit$s, 1, function(x) sqrt(sum((x - constant_s[1,])^2)))
  expect_true(all(abs(fit$lambda - lambda) < 1e-8))

  dist_ind <-
    ifelse(f, (abs(z) - cut)^2, 0) +
    (x[,2] - .1)^2 +
    (x[,3] - .2)^2 +
    (x[,4] - .3)^2

  expect_true(all(abs(fit$dist_ind - dist_ind) < 1e-10))

  expect_true(abs(sum(dist_ind) - fit$dist) < 1e-10)
})



test_that("Expect project_to_curve to error elegantly", {
  expect_error(project_to_curve(list(1), list(1)))
  expect_error(project_to_curve(x = list(), s = s, stretch = 0))
  expect_error(project_to_curve(x = x, s = list(), stretch = 0))
  expect_error(project_to_curve(x, s, stretch = -1), "larger than or equal to 0")
  expect_error(project_to_curve(x, s, stretch = "10"))
  expect_error(project_to_curve(x, cbind(s, s)), "must have an equal number of columns")
})



test_that("Projecting to random data produces correct results", {
  for (i in seq_len(10)) {
    s <- matrix(runif(100), ncol = 2)
    x <- matrix(runif(100), ncol = 2)

    fit <- project_to_curve(
      x = x,
      s = s,
      stretch = 0
    )

    test_projection(x, s, stretch = 0, fit)
  }
})

#' Fit a Principal Curve
#'
#' Fits a principal curve which describes a smooth curve that passes through the \code{middle}
#' of the data \code{x} in an orthogonal sense.  This curve is a nonparametric generalization
#' of a linear principal component.  If a closed curve is fit (using \code{smoother = "periodic_lowess"})
#' then the starting curve defaults to a circle, and each fit is followed by a bias correction
#' suggested by Jeff Banfield.
#'
#' @param x a matrix of points in arbitrary dimension.
#' @param start either a previously fit principal curve, or else a matrix
#'   of points that in row order define a starting curve. If missing or NULL,
#'   then the first principal component is used.  If the smoother is
#'   \code{"periodic_lowess"}, then a circle is used as the start.
#' @param thresh convergence threshold on shortest distances to the curve.
#' @param plot_iterations If \code{TRUE} the iterations are plotted.
#' @param maxit maximum number of iterations.
#' @param stretch A stretch factor for the endpoints of the curve,
#'   allowing the curve to grow to avoid bunching at the end.
#'   Must be a numeric value between 0 and 2.
#' @param smoother choice of smoother. The default is
#'   \code{"smooth_spline"}, and other choices are \code{"lowess"} and
#'   \code{"periodic_lowess"}. The latter allows one to fit closed curves.
#'   Beware, you may want to use \code{iter = 0} with \code{lowess()}.
#' @param approx_points Approximate curve after smoothing to reduce computational time.
#'   If \code{FALSE}, no approximation of the curve occurs. Otherwise,
#'   \code{approx_points} must be equal to the number of points the curve
#'   gets approximated to; preferably about 100.
#' @param trace If \code{TRUE}, the iteration information is printed
#' @param ... additional arguments to the smoothers
#'
#' @return An object of class \code{"principal_curve"} is returned. For this object
#'   the following generic methods a currently available: \code{plot, points, lines}.
#'
#'   It has components:
#'     \item{s}{a matrix corresponding to \code{x}, giving their projections
#'       onto the curve.}
#'   \item{ord}{an index, such that \code{s[order, ]} is smooth.}
#'   \item{lambda}{for each point, its arc-length from the beginning of the
#'     curve. The curve is parametrized approximately by arc-length, and
#'     hence is \code{unit-speed}.}
#'   \item{dist}{the sum-of-squared distances from the points to their
#'     projections.}
#'   \item{converged}{A logical indicating whether the algorithm converged
#'     or not.}
#'   \item{num_iterations}{Number of iterations completed before returning.}
#'   \item{call}{the call that created this object; allows it to be
#'     \code{updated()}.}
#'
#' @seealso \code{\link{project_to_curve}}
#'
#' @keywords regression smooth nonparametric
#'
#' @references
#'  Hastie, T. and Stuetzle, W.,
#'  \href{https://www.jstor.org/stable/2289936}{Principal Curves},
#'  JASA, Vol. 84, No. 406 (Jun., 1989), pp. 502-516,
#'  DOI: \href{https://doi.org/10.2307/2289936}{10.2307/2289936}
#'  (\href{https://web.stanford.edu/~hastie/Papers/principalcurves.pdf}{PDF}).
#'
#' @export
#'
#' @include smoother_functions.R
#'
#' @importFrom stats lowess smooth.spline predict var
#' @importFrom grDevices extendrange
#'
#' @examples
#' x <- runif(100,-1,1)
#' x <- cbind(x, x ^ 2 + rnorm(100, sd = 0.1))
#' fit <- principal_curve(x)
#' plot(fit)
#' lines(fit)
#' points(fit)
#' whiskers(x, fit$s)
principal_curve <- function(
  x,
  start = NULL,
  thresh = 0.001,
  maxit = 10,
  stretch = 2,
  smoother = names(smoother_functions),
  approx_points = FALSE,
  trace = FALSE,
  plot_iterations = FALSE,
  ...
) {
  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  # Validate arguments:
  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  # Check 'x'
  if (!is.matrix(x)) {
    stop("Argument ", sQuote("x"), " must be a matrix.")
  }

  # Check 'smoother'
  if (is.function(smoother)) {
    smoother_function <- smoother
    bias_correct_curve <- FALSE
  } else if (is.character(smoother)) {
    # substitute .'s to _'s for backwards compatibility
    smoother <- gsub(".", "_", smoother, fixed = TRUE)
    smoother <- match.arg(smoother)
    smoother_function <- smoother_functions[[smoother]]

    if (smoother == "periodic_lowess") {
      stretch <- 0
      bias_correct_curve <- TRUE
    } else {
      bias_correct_curve <- FALSE
    }
  }

  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  # Setup
  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  function_call <- match.call()
  dist_old <- sum(diag(stats::var(x)))

  # You can give starting values for the curve
  if (missing(start) || is.null(start)) {
    # use largest principal component
    if (is.character(smoother) && smoother == "periodic_lowess") {
      start <- start_circle(x)
    } else {
      xbar <- colMeans(x)
      xstar <- scale(x, center = xbar, scale=FALSE)
      svd_xstar <- svd(xstar)
      dd <- svd_xstar$d
      lambda <- svd_xstar$u[,1] * dd[1]
      ord <- order(lambda)
      s <- scale(outer(lambda, svd_xstar$v[,1]), center = -xbar, scale = FALSE)
      dimnames(s) <- dimnames(x)
      dist <- sum((dd^2)[-1]) * nrow(x)
      start <- list(s = s, ord = ord, lambda = lambda, dist = dist)
    }
  } else if (!inherits(start, "principal_curve")) {
    # use given starting curve
    if (is.matrix(start)) {
      start <- project_to_curve(x = x, s = start, stretch = stretch)
    } else {
      stop("Invalid starting curve: should be a matrix or principal_curve")
    }
  }

  pcurve <- start

  if (plot_iterations) {
    plot(
      x[,1:2],
      xlim = grDevices::extendrange(x[,1], f = .2),
	    ylim = grDevices::extendrange(x[,2], f = .2)
    )
    lines(pcurve$s[pcurve$ord, 1:2])
  }

  it <- 0
  if (trace) {
    cat("Starting curve---distance^2: ", pcurve$dist, "\n", sep="")
  }

  # Pre-allocate nxp matrix 's'
  s <- matrix(
    as.double(NA),
    nrow = ifelse(approx_points > 0, approx_points, nrow(x)),
    ncol = ncol(x),
    dimnames = list(NULL, colnames(x))
  )

  has_converged <- abs(dist_old - pcurve$dist) <= thresh * dist_old
  while (!has_converged && it < maxit) {
    it <- it + 1

    if (approx_points > 0) {
      sort_lambda <- sort(pcurve$lambda)
      xout_lambda <- seq(sort_lambda[[1]], sort_lambda[[length(sort_lambda)]], length.out = approx_points)
    }

    for (jj in seq_len(ncol(x))) {
      # Smooth (lambda, x)
      yjj <- smoother_function(pcurve$lambda, x[,jj], ...)

      # If requested, approximate the smoothed curve to reduce computational complexity
      if (approx_points > 0) {
        yjj <- stats::approx(x = sort_lambda, y = yjj, xout = xout_lambda, ties = "ordered")$y
      }

      # Store curve
      s[,jj] <- yjj
    }

    dist_old <- pcurve$dist

    # Finds the "projection index" for a matrix of points 'x',
    pcurve <- project_to_curve(x = x, s = s, stretch = stretch)

    # Bias correct
    if (bias_correct_curve) {
      pcurve <- bias_correct_curve(x = x, pcurve = pcurve, ...)
    }

    # Converged
    has_converged <- abs(dist_old - pcurve$dist) <= thresh * dist_old

    if (plot_iterations) {
      plot(
        x[,1:2],
        xlim = grDevices::extendrange(x[,1], f = .2),
        ylim = grDevices::extendrange(x[,2], f = .2)
      )
      lines(pcurve$s[pcurve$ord, 1:2])
    }

    if (trace) {
      cat("Iteration ", it, "---distance^2: ", pcurve$dist, "\n", sep="")
    }
  }

  # Return fit
  out <- list(
    s = pcurve$s,
    ord = pcurve$ord,
    lambda = pcurve$lambda,
    dist = pcurve$dist,
    converged = has_converged,
    num_iterations = as.integer(it),
    call = function_call
  )
  class(out) <- "principal_curve"
  out
}

formals(principal_curve)$smoother <- names(smoother_functions)

#' @rdname principal_curve
#' @export
#' @importFrom graphics lines
lines.principal_curve <- function(x, ...) {
  if ("ord" %in% names(x)) {
    ord <- x[["ord"]]
  } else {
    ord <- x[["tag"]]
  }
  graphics::lines(x$s[ord, ], ...)
}

#' @rdname principal_curve
#' @export
#' @importFrom graphics plot
plot.principal_curve <- function(x, ...) {
  if ("ord" %in% names(x)) {
    ord <- x[["ord"]]
  } else {
    ord <- x[["tag"]]
  }
  graphics::plot(x$s[ord, ], ..., type = "l")
}

#' @rdname principal_curve
#' @export
#' @importFrom graphics points
points.principal_curve <- function(x, ...) {
  graphics::points(x$s, ...)
}

#' @rdname principal_curve
#' @param s a parametrized curve, represented by a polygon.
#' @importFrom graphics segments
#' @export
whiskers <- function(x, s, ...) {
  graphics::segments(x[, 1], x[, 2], s[, 1], s[, 2], ...)
}

context("Testing principal_curve")

z <- seq(-1, 1, length.out = 100)
s <- cbind(z, z^2, z^3, z^4)
x <- s + rnorm(length(s), mean = 0, sd = .005)

file <- tempfile(fileext = ".svg")
on.exit(unlink(file))

test_that("Testing principal_curve with smooth_spline", {
  svg(file, 5, 5)
  sink(file)
  fit <- principal_curve(x, smoother = "smooth_spline", plot_iterations = TRUE, trace = TRUE)
  sink()
  dev.off()

  svg(file, 5, 5)
  expect_error({
    plot(fit)
    points(fit)
    lines(fit)
    whiskers(x = x, s = fit$s)
  }, NA)
  dev.off()

  expect_gte(cor(as.vector(fit$s), as.vector(s)), .99)
  expect_gte(abs(cor(fit$ord, seq_len(100))), .99)
})


test_that("Testing principal_curve returns same results with approx optimisation", {
  fit <- principal_curve(x)
  fit2 <- principal_curve(x, approx_points = 66)

  expect_gte(cor(as.vector(fit$s), as.vector(fit2$s)), .99)
  expect_gte(abs(cor(order(fit$ord), order(fit2$ord))), .99)
  expect_gte(abs(cor(fit$lambda, fit2$lambda)), .99)
})

test_that("Testing principal_curve with custom function", {
  fun <- function(lambda, xj, ...) {
    stats::lowess(lambda, xj, ...)$y
  }

  svg(file, 5, 5)
  fit <- principal_curve(x, smoother = fun, plot_iterations = TRUE)
  dev.off()

  svg(file, 5, 5)
  expect_error({
    plot(fit)
    points(fit)
  }, NA)
  dev.off()

  expect_gte(cor(as.vector(fit$s), as.vector(s)), .99)
  expect_gte(abs(cor(fit$ord, seq_len(100))), .99)
})


test_that("Testing principal_curve with a given start curve", {
  start <- matrix(c(0, 0, 0, 0, 1, 1, 1, 1), ncol = 4, byrow = TRUE)

  svg(file, 5, 5)
  fit <- principal_curve(x, smoother = "smooth_spline", start = start, plot_iterations = TRUE)
  dev.off()

  svg(file, 5, 5)
  expect_error({
    plot(fit)
    points(fit)
  }, NA)
  dev.off()

  expect_gte(cor(as.vector(fit$s), as.vector(s)), .99)
  expect_gte(abs(cor(fit$ord, seq_len(100))), .99)
})

test_that("Expect principal_curve to error elegantly", {
  expect_error(principal_curve(list(1)))
  expect_error(principal_curve(x, stretch = -1), "larger than or equal to 0")
  expect_error(principal_curve(x, stretch = "10"))
  expect_error(principal_curve(x, start = "10"), "should be a matrix or principal_curve")
})

test_that("Testing principal_curve with lowess", {
  fit <- principal_curve(x, smoother = "lowess")

  svg(file, 5, 5)
  expect_error({
    plot(fit)
    points(fit)
  }, NA)
  dev.off()

  expect_gte(cor(as.vector(fit$s), as.vector(s)), .99)
  expect_gte(abs(cor(fit$ord, seq_len(100))), .99)
})

z <- seq(0, 2 * pi, length.out = 100)
s <- cbind(cos(z), sin(z), 2 * cos(z), 3 * sin(z))
x <- s + rnorm(length(s), mean = 0, sd = .005)

test_that("Testing principal_curve with periodic_lowess", {
  fit <- principal_curve(x, smoother = "periodic_lowess")

  svg(file, 5, 5)
  expect_error({
    plot(fit)
    points(fit)
  }, NA)
  dev.off()

  expect_gte(cor(as.vector(fit$s), as.vector(s)), .99)
})

test_that("principal_curve does not produce NAs when checking for convergence", {
  mat <- matrix(rep(seq(1, 11, 1), 3), ncol = 3)
  fit <- principal_curve(mat)
  expect_lte(sum((fit$s - mat)^2), 1e-10)
})



test_that("principal_curve keeps rownames and colnames", {
  mat <- matrix(runif(100), ncol = 4)
  rownames(mat) <- seq_len(25)
  fit <- principal_curve(mat)
  expect_equal(dimnames(fit$s), dimnames(mat))

  colnames(mat) <- LETTERS[1:4]
  fit <- principal_curve(mat)
  expect_equal(dimnames(fit$s), dimnames(mat))

  rownames(mat) <- NULL
  fit <- principal_curve(mat)
  expect_equal(dimnames(fit$s), dimnames(mat))

  colnames(mat) <- NULL
  fit <- principal_curve(mat)
  expect_equal(dimnames(fit$s), dimnames(mat))
})

context("Testing smoother_functions")

for (nsf in names(smoother_functions)) {
  test_that(paste0("Smoother function ", nsf, " works as expected"), {
    sf <- smoother_functions[[nsf]]

    lambda <- seq(-1, 1, length.out = 100)
    xj <- lambda^2 + rnorm(length(lambda), 0, .1)

    # check whether y is smoother than xj
    y <- sf(lambda, xj)
    expect_lte(mean(diff(y)^2), mean(diff(xj)^2))

    # check whether shuffled input data results in the same output
    or <- sample.int(length(lambda))
    y2 <- sf(lambda[or], xj[or])
    expect_lte(mean(diff(y)^2), mean(diff(xj)^2))
  })
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{princurve-package}
\alias{princurve-package}
\alias{princurve}
\title{Fits a Principal Curve in Arbitrary Dimension}
\description{
Fits a Principal Curve in Arbitrary Dimension
}
\references{
Hastie, T. and Stuetzle, W.,
 \href{https://www.jstor.org/stable/2289936}{Principal Curves},
 JASA, Vol. 84, No. 406 (Jun., 1989), pp. 502-516,
 DOI: \href{https://doi.org/10.2307/2289936}{10.2307/2289936}
 (\href{https://web.stanford.edu/~hastie/Papers/principalcurves.pdf}{PDF}).

 See also Banfield and Raftery (JASA, 1992).
}
\seealso{
\code{\link{principal_curve}}, \code{\link{project_to_curve}}
}
\keyword{nonparametric}
\keyword{regression}
\keyword{smooth}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{project_to_curve}
\alias{project_to_curve}
\title{Project a set of points to the closest point on a curve}
\usage{
project_to_curve(x, s, stretch = 2)
}
\arguments{
\item{x}{a matrix of data points.}

\item{s}{a parametrized curve, represented by a polygon.}

\item{stretch}{A stretch factor for the endpoints of the curve,
allowing the curve to grow to avoid bunching at the end.
Must be a numeric value between 0 and 2.}
}
\value{
A structure is returned which represents a fitted curve.  It has components
  \item{s}{The fitted points on the curve corresponding to each point \code{x}}
  \item{ord}{the order of the fitted points}
  \item{lambda}{The projection index for each point}
  \item{dist}{The total squared distance from the curve}
  \item{dist_ind}{The squared distances from the curve to each of the respective points}
}
\description{
Finds the projection index for a matrix of points \code{x}, when
projected onto a curve \code{s}. The curve need not be of the same
length as the number of points.
}
\seealso{
\code{\link{principal_curve}}
}
\keyword{nonparametric}
\keyword{regression}
\keyword{smooth}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smoother_functions.R
\docType{data}
\name{smoother_functions}
\alias{smoother_functions}
\title{Smoother functions}
\format{An object of class \code{list} of length 3.}
\usage{
smoother_functions
}
\description{
Each of these functions have an interface \code{function(lambda, xj, ...)}, and
return smoothed values for xj. The output is expected to be ordered along an ordered lambda.
This means that the following is true:
\preformatted{
x <- runif(100)
y <- runif(100)
ord <- sample.int(100)
sfun <- smoother_functions[[1]]
all(sfun(x, y) == sfun(x[ord], y[ord]))
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/principal_curve.R
\name{principal_curve}
\alias{principal_curve}
\alias{lines.principal_curve}
\alias{plot.principal_curve}
\alias{points.principal_curve}
\alias{whiskers}
\title{Fit a Principal Curve}
\usage{
principal_curve(x, start = NULL, thresh = 0.001, maxit = 10,
  stretch = 2, smoother = c("smooth_spline", "lowess",
  "periodic_lowess"), approx_points = FALSE, trace = FALSE,
  plot_iterations = FALSE, ...)

\method{lines}{principal_curve}(x, ...)

\method{plot}{principal_curve}(x, ...)

\method{points}{principal_curve}(x, ...)

whiskers(x, s, ...)
}
\arguments{
\item{x}{a matrix of points in arbitrary dimension.}

\item{start}{either a previously fit principal curve, or else a matrix
of points that in row order define a starting curve. If missing or NULL,
then the first principal component is used.  If the smoother is
\code{"periodic_lowess"}, then a circle is used as the start.}

\item{thresh}{convergence threshold on shortest distances to the curve.}

\item{maxit}{maximum number of iterations.}

\item{stretch}{A stretch factor for the endpoints of the curve,
allowing the curve to grow to avoid bunching at the end.
Must be a numeric value between 0 and 2.}

\item{smoother}{choice of smoother. The default is
\code{"smooth_spline"}, and other choices are \code{"lowess"} and
\code{"periodic_lowess"}. The latter allows one to fit closed curves.
Beware, you may want to use \code{iter = 0} with \code{lowess()}.}

\item{approx_points}{Approximate curve after smoothing to reduce computational time.
If \code{FALSE}, no approximation of the curve occurs. Otherwise,
\code{approx_points} must be equal to the number of points the curve
gets approximated to; preferably about 100.}

\item{trace}{If \code{TRUE}, the iteration information is printed}

\item{plot_iterations}{If \code{TRUE} the iterations are plotted.}

\item{...}{additional arguments to the smoothers}

\item{s}{a parametrized curve, represented by a polygon.}
}
\value{
An object of class \code{"principal_curve"} is returned. For this object
  the following generic methods a currently available: \code{plot, points, lines}.

  It has components:
    \item{s}{a matrix corresponding to \code{x}, giving their projections
      onto the curve.}
  \item{ord}{an index, such that \code{s[order, ]} is smooth.}
  \item{lambda}{for each point, its arc-length from the beginning of the
    curve. The curve is parametrized approximately by arc-length, and
    hence is \code{unit-speed}.}
  \item{dist}{the sum-of-squared distances from the points to their
    projections.}
  \item{converged}{A logical indicating whether the algorithm converged
    or not.}
  \item{num_iterations}{Number of iterations completed before returning.}
  \item{call}{the call that created this object; allows it to be
    \code{updated()}.}
}
\description{
Fits a principal curve which describes a smooth curve that passes through the \code{middle}
of the data \code{x} in an orthogonal sense.  This curve is a nonparametric generalization
of a linear principal component.  If a closed curve is fit (using \code{smoother = "periodic_lowess"})
then the starting curve defaults to a circle, and each fit is followed by a bias correction
suggested by Jeff Banfield.
}
\examples{
x <- runif(100,-1,1)
x <- cbind(x, x ^ 2 + rnorm(100, sd = 0.1))
fit <- principal_curve(x)
plot(fit)
lines(fit)
points(fit)
whiskers(x, fit$s)
}
\references{
Hastie, T. and Stuetzle, W.,
 \href{https://www.jstor.org/stable/2289936}{Principal Curves},
 JASA, Vol. 84, No. 406 (Jun., 1989), pp. 502-516,
 DOI: \href{https://doi.org/10.2307/2289936}{10.2307/2289936}
 (\href{https://web.stanford.edu/~hastie/Papers/principalcurves.pdf}{PDF}).
}
\seealso{
\code{\link{project_to_curve}}
}
\keyword{nonparametric}
\keyword{regression}
\keyword{smooth}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/start_circle.R
\name{start_circle}
\alias{start_circle}
\title{Generate circle as initial curve}
\usage{
start_circle(x)
}
\arguments{
\item{x}{The data for which to generate the initial circle}
}
\description{
The starting circle is defined in the first two dimensions,
and has zero values in all other dimensions.
}
\examples{
\dontrun{
x <- cbind(
  rnorm(100, 1, .2),
  rnorm(100, -5, .2),
  runif(100, 1.9, 2.1),
  runif(100, 2.9, 3.1)
)
circ <- start_circle(x)
plot(x)
lines(circ)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{principal.curve}
\alias{principal.curve}
\alias{lines.principal.curve}
\alias{plot.principal.curve}
\alias{points.principal.curve}
\alias{get.lam}
\title{Deprecated functions}
\usage{
principal.curve(...)

\method{lines}{principal.curve}(...)

\method{plot}{principal.curve}(...)

\method{points}{principal.curve}(...)

get.lam(...)
}
\arguments{
\item{...}{Catch-all for old parameters.}
}
\description{
This function is deprecated, please use
\code{\link{principal_curve}} and \code{\link{project_to_curve}} instead.
}
